@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (0, 11, 47)
-CODENAME = "the grid"
-BUILD_DT = (2021, 7, 22)
+VERSION = (0, 12, 8)
+CODENAME = "fil\033[33med"
+BUILD_DT = (2021, 8, 1)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
