@@ -12,6 +12,7 @@ ebi('ops').innerHTML = (
 	'<a href="#" data-dest="" tt="close submenu">---</a>\n' +
 	(have_up2k_idx ? (
 		'<a href="#" data-perm="read" data-dest="search" tt="search for files by attributes, path/name, music tags, or any combination of those.$N$N&lt;code&gt;foo bar&lt;/code&gt; = must contain both foo and bar,$N&lt;code&gt;foo -bar&lt;/code&gt; = must contain foo but not bar,$N&lt;code&gt;^yana .opus$&lt;/code&gt; = must start with yana and have the opus extension">🔎</a>\n' +
+		(have_del && have_unpost ? '<a href="#" data-dest="unpost" tt="unpost: delete your recent uploads">🧯</a>\n' : '') +
 		'<a href="#" data-dest="up2k" tt="up2k: upload files (if you have write-access) or toggle into the search-mode and drag files onto the search button to see if they exist somewhere on the server">🚀</a>\n'
 	) : (
 		'<a href="#" data-perm="write" data-dest="up2k" tt="up2k: upload files with resume support (close your browser and drop the same files in later)">🚀</a>\n'
@@ -29,15 +30,20 @@ ebi('ops').innerHTML = (
 // media player
 ebi('widget').innerHTML = (
 	'<div id="wtoggle">' +
-	'<span id="wzip"><a' +
-	' href="#" id="selall" tt="select all files">sel.<br />all</a><a' +
+	'<span id="wfm"><a' +
+	' href="#" id="fren" tt="rename selected item$NHotkey: F2">✎<span>name</span></a><a' +
+	' href="#" id="fdel" tt="delete selected items$NHotkey: ctrl-K">⌫<span>delete</span></a><a' +
+	' href="#" id="fcut" tt="cut selected items &lt;small&gt;(then paste somewhere else)&lt;/small&gt;$NHotkey: ctrl-X">✂<span>cut</span></a><a' +
+	' href="#" id="fpst" tt="paste a previously cut/copied selection$NHotkey: ctrl-V">📋<span>paste</span></a>' +
+	'</span><span id="wzip"><a' +
+	' href="#" id="selall" tt="select all files$NHotkey: ctrl-A (when file focused)">sel.<br />all</a><a' +
 	' href="#" id="selinv" tt="invert selection">sel.<br />inv.</a><a' +
 	' href="#" id="selzip" tt="download selection as archive">zip</a>' +
 	'</span><span id="wnp"><a' +
 	' href="#" id="npirc" tt="copy irc-formatted track info">📋irc</a><a' +
 	' href="#" id="nptxt" tt="copy plaintext track info">📋txt</a>' +
 	'</span><a' +
-	'	href="#" id="wtgrid">田</a><a' +
+	'	href="#" id="wtgrid" tt="toggle grid/list view">田</a><a' +
 	'	href="#" id="wtico">♫</a>' +
 	'</div>' +
 	'<div id="widgeti">' +
@@ -127,6 +133,7 @@ ebi('op_cfg').innerHTML = (
 	'	<div>\n' +
 	'		<a id="tooltips" class="tgl btn" href="#" tt="◔ ◡ ◔">ℹ️ tooltips</a>\n' +
 	'		<a id="lightmode" class="tgl btn" href="#">☀️ lightmode</a>\n' +
+	'		<a id="dotfiles" class="tgl btn" href="#" tt="show hidden files (if server permits)">dotfiles</a>\n' +
 	'		<a id="griden" class="tgl btn" href="#" tt="toggle icons or list-view$NHotkey: G">田 the grid</a>\n' +
 	'		<a id="thumbs" class="tgl btn" href="#" tt="in icon view, toggle icons or thumbnails$NHotkey: T">🖼️ thumbs</a>\n' +
 	'	</div>\n' +
@@ -146,7 +153,7 @@ ebi('op_cfg').innerHTML = (
 );
 
 
-// tree sidebar
+// navpane
 ebi('tree').innerHTML = (
 	'<div id="treeh">\n' +
 	'	<a href="#" id="detree" tt="show breadcrumbs$NHotkey: B">🍞...</a>\n' +
@@ -208,17 +215,6 @@ function goto(dest) {
 }
 
 
-(function () {
-	goto();
-	var op = sread('opmode');
-	if (op !== null && op !== '.')
-		try {
-			goto(op);
-		}
-		catch (ex) { }
-})();
-
-
 var have_webp = null;
 (function () {
 	var img = new Image();
@@ -280,7 +276,7 @@ var mpl = (function () {
 		r.os_ctl = !r.os_ctl && have_mctl;
 		bcfg_set('au_os_ctl', r.os_ctl);
 		if (!have_mctl)
-			alert('need firefox 82+ or chrome 73+');
+			toast.err(5, 'need firefox 82+ or chrome 73+');
 	};
 
 	ebi('au_osd_cv').onclick = function (e) {
@@ -526,15 +522,14 @@ var mp = new MPlayer();
 makeSortable(ebi('files'), mp.read_order.bind(mp));
 
 
-function get_np() {
+function ft2dict(tr) {
 	var th = ebi('files').tHead.rows[0].cells,
-		tr = QS('#files tr.play').cells,
 		rv = [],
 		ra = [],
 		rt = {};
 
 	for (var a = 1, aa = th.length; a < aa; a++) {
-		var tv = tr[a].textContent,
+		var tv = tr.cells[a].textContent,
 			tk = a == 1 ? 'file' : th[a].getAttribute('name').split('/').slice(-1)[0],
 			vis = th[a].className.indexOf('min') === -1;
 
@@ -545,6 +540,12 @@ function get_np() {
 		rt[tk] = tv;
 	}
 	return [rt, rv, ra];
+}
+
+
+function get_np() {
+	var tr = QS('#files tr.play');
+	return ft2dict(tr);
 };
 
 
@@ -1348,7 +1349,7 @@ function play(tid, is_ev, seek, call_depth) {
 		return true;
 	}
 	catch (ex) {
-		alert('playback failed: ' + ex);
+		toast.err(0, 'playback failed: ' + ex);
 	}
 	setclass(oid, 'play');
 	setTimeout(next_song, 500);
@@ -1459,6 +1460,301 @@ function play_linked() {
 })();
 
 
+var fileman = (function () {
+	var bren = ebi('fren'),
+		bdel = ebi('fdel'),
+		bcut = ebi('fcut'),
+		bpst = ebi('fpst'),
+		r = {};
+
+	r.clip = null;
+	r.bus = new BroadcastChannel("fileman_bus");
+
+	r.render = function () {
+		if (r.clip === null)
+			r.clip = jread('fman_clip', []);
+
+		var nsel = msel.getsel().length;
+		clmod(bren, 'en', nsel == 1);
+		clmod(bdel, 'en', nsel);
+		clmod(bcut, 'en', nsel);
+		clmod(bpst, 'en', r.clip && r.clip.length);
+		bren.style.display = have_mv && has(perms, 'write') && has(perms, 'move') ? '' : 'none';
+		bdel.style.display = have_del && has(perms, 'delete') ? '' : 'none';
+		bcut.style.display = have_mv && has(perms, 'move') ? '' : 'none';
+		bpst.style.display = have_mv && has(perms, 'write') ? '' : 'none';
+		bpst.setAttribute('tt', 'paste ' + r.clip.length + ' items$NHotkey: ctrl-V');
+		ebi('wfm').style.display = QS('#wfm a.en:not([display])') ? '' : 'none';
+	};
+
+	r.rename = function (e) {
+		ev(e);
+		if (bren.style.display)
+			return toast.err(3, 'cannot rename:\nyou do not have “move” permission in this folder');
+
+		var sel = msel.getsel();
+		if (sel.length !== 1)
+			return toast.err(3, 'select exactly 1 item to rename');
+
+		var src = sel[0].vp;
+		if (src.endsWith('/'))
+			src = src.slice(0, -1);
+
+		var vsp = vsplit(src),
+			base = vsp[0],
+			ofn = uricom_dec(vsp[1])[0];
+
+		var rui = ebi('rui');
+		if (!rui) {
+			rui = mknod('div');
+			rui.setAttribute('id', 'rui');
+			document.body.appendChild(rui);
+		}
+		var html = [
+			'<h1>rename file</h1>',
+			'<div><table>',
+			'<tr><td>old:</td><td><input type="text" id="rn_old" readonly /></td></tr>',
+			'<tr><td>new:</td><td><input type="text" id="rn_new" /></td></tr>',
+			'</table></div>',
+			'<div>',
+			'<button id="rn_dec">url-decode</button>',
+			'|',
+			'<button id="rn_reset">↺ reset</button>',
+			'<button id="rn_cancel">❌ cancel</button>',
+			'<button id="rn_apply">✅ apply rename</button>',
+			'</div>',
+			'<div><table>'
+		];
+
+		var vars = ft2dict(ebi(sel[0].id).closest('tr')),
+			keys = vars[1].concat(vars[2]);
+
+		vars = vars[0];
+		for (var a = 0; a < keys.length; a++)
+			html.push('<tr><td>' + esc(keys[a]) + '</td><td><input type="text" readonly value="' + esc(vars[keys[a]]) + '" /></td></tr>');
+
+		html.push('</table></div>');
+		rui.innerHTML = html.join('\n');
+		var iold = ebi('rn_old'),
+			inew = ebi('rn_new');
+
+		function rn_reset() {
+			inew.value = iold.value;
+			inew.focus();
+			inew.setSelectionRange(0, inew.value.lastIndexOf('.'), "forward");
+		}
+		function rn_cancel() {
+			rui.parentNode.removeChild(rui);
+		}
+
+		inew.onkeydown = function (e) {
+			if (e.key == 'Escape')
+				return rn_cancel();
+
+			if (e.key == 'Enter')
+				return rn_apply();
+		};
+		ebi('rn_cancel').onclick = rn_cancel;
+		ebi('rn_reset').onclick = rn_reset;
+		ebi('rn_apply').onclick = rn_apply;
+		ebi('rn_dec').onclick = function () {
+			inew.value = uricom_dec(inew.value)[0];
+		};
+
+		iold.value = ofn;
+		rn_reset();
+
+		function rn_apply() {
+			var dst = base + uricom_enc(inew.value, false);
+
+			function rename_cb() {
+				if (this.readyState != XMLHttpRequest.DONE)
+					return;
+
+				if (this.status !== 200) {
+					var msg = this.responseText;
+					toast.err(9, 'rename failed:\n' + msg);
+					return;
+				}
+				toast.ok(2, 'rename OK');
+				treectl.goto(get_evpath());
+				rn_cancel();
+			}
+			var xhr = new XMLHttpRequest();
+			xhr.open('GET', src + '?move=' + dst, true);
+			xhr.onreadystatechange = rename_cb;
+			xhr.send();
+		};
+	};
+
+	r.delete = function (e) {
+		ev(e);
+		if (bdel.style.display)
+			return toast.err(3, 'cannot delete:\nyou do not have “delete” permission in this folder');
+
+		var sel = msel.getsel(),
+			vps = [];
+
+		for (var a = 0; a < sel.length; a++)
+			vps.push(sel[a].vp);
+
+		if (!sel.length)
+			return toast.err(3, 'select at least 1 item to delete');
+
+		if (!confirm('===== DANGER =====\nDELETE these ' + vps.length + ' items?\n\n' + vps.join('\n')))
+			return;
+
+		if (!confirm('Last chance! Delete?'))
+			return;
+
+		function deleter() {
+			var xhr = new XMLHttpRequest(),
+				vp = vps.shift();
+
+			if (!vp) {
+				toast.ok(2, 'delete OK');
+				treectl.goto(get_evpath());
+				return;
+			}
+			toast.inf(0, 'deleting ' + (vps.length + 1) + ' items\n\n' + vp);
+
+			xhr.open('GET', vp + '?delete', true);
+			xhr.onreadystatechange = delete_cb;
+			xhr.send();
+		}
+		function delete_cb() {
+			if (this.readyState != XMLHttpRequest.DONE)
+				return;
+
+			if (this.status !== 200) {
+				var msg = this.responseText;
+				toast.err(9, 'delete failed:\n' + msg);
+				return;
+			}
+			deleter();
+		}
+		deleter();
+	};
+
+	r.cut = function (e) {
+		ev(e);
+		if (bcut.style.display)
+			return toast.err(3, 'cannot cut:\nyou do not have “move” permission in this folder');
+
+		var sel = msel.getsel(),
+			vps = [];
+
+		if (!sel.length)
+			return toast.err(3, 'select at least 1 item to cut');
+
+		for (var a = 0; a < sel.length; a++) {
+			vps.push(sel[a].vp);
+			var cl = ebi(sel[a].id).closest('tr').classList,
+				inv = cl.contains('c1');
+
+			cl.remove(inv ? 'c1' : 'c2');
+			cl.add(inv ? 'c2' : 'c1');
+		}
+
+		toast.inf(1, 'cut ' + sel.length + ' items');
+		jwrite('fman_clip', vps);
+		r.tx(1);
+	};
+
+	r.paste = function (e) {
+		ev(e);
+		if (bpst.style.display)
+			return toast.err(3, 'cannot paste:\nyou do not have “write” permission in this folder');
+
+		if (!r.clip.length)
+			return toast.err(5, 'first cut some files/folders to paste\n\nnote: you can cut/paste across different browser tabs');
+
+		var req = [],
+			exists = [],
+			indir = [],
+			srcdir = vsplit(r.clip[0])[0],
+			links = QSA('#files tbody td:nth-child(2) a');
+
+		for (var a = 0, aa = links.length; a < aa; a++)
+			indir.push(vsplit(links[a].getAttribute('href'))[1]);
+
+		for (var a = 0; a < r.clip.length; a++) {
+			var found = false;
+			for (var b = 0; b < indir.length; b++) {
+				if (r.clip[a].endsWith('/' + indir[b])) {
+					exists.push(r.clip[a]);
+					found = true;
+				}
+			}
+			if (!found)
+				req.push(r.clip[a]);
+		}
+
+		if (exists.length)
+			alert('these ' + exists.length + ' items cannot be pasted here (names already exist):\n\n' + uricom_adec(exists).join('\n'));
+
+		if (!req.length)
+			return;
+
+		if (!confirm('paste these ' + req.length + ' items here?\n\n' + uricom_adec(req).join('\n')))
+			return;
+
+		function paster() {
+			var xhr = new XMLHttpRequest(),
+				vp = req.shift();
+
+			if (!vp) {
+				toast.ok(2, 'paste OK');
+				treectl.goto(get_evpath());
+				r.tx(srcdir);
+				return;
+			}
+			toast.inf(0, 'pasting ' + (req.length + 1) + ' items\n\n' + uricom_dec(vp)[0]);
+
+			var dst = get_evpath() + vp.split('/').slice(-1)[0];
+
+			xhr.open('GET', vp + '?move=' + dst, true);
+			xhr.onreadystatechange = paste_cb;
+			xhr.send();
+		}
+		function paste_cb() {
+			if (this.readyState != XMLHttpRequest.DONE)
+				return;
+
+			if (this.status !== 200) {
+				var msg = this.responseText;
+				toast.err(9, 'paste failed:\n' + msg);
+				return;
+			}
+			paster();
+		}
+		paster();
+
+		jwrite('fman_clip', []);
+	};
+
+	r.bus.onmessage = function (e) {
+		r.clip = null;
+		r.render();
+		var me = get_evpath();
+		if (e && e.data == me)
+			treectl.goto(e.data);
+	};
+
+	r.tx = function (msg) {
+		r.bus.postMessage(msg);
+		r.bus.onmessage();
+	};
+
+	bren.onclick = r.rename;
+	bdel.onclick = r.delete;
+	bcut.onclick = r.cut;
+	bpst.onclick = r.paste;
+
+	return r;
+})();
+
+
 var thegrid = (function () {
 	var lfiles = ebi('files'),
 		gfiles = mknod('div');
@@ -1497,9 +1793,6 @@ var thegrid = (function () {
 
 	ebi('griden').onclick = ebi('wtgrid').onclick = function (e) {
 		ev(e);
-		if (!this.closest)
-			return;
-
 		r.en = !r.en;
 		bcfg_set('griden', r.en);
 		if (r.en) {
@@ -1785,10 +2078,11 @@ function tree_up() {
 
 
 document.onkeydown = function (e) {
-	if (!document.activeElement || document.activeElement != document.body && document.activeElement.nodeName.toLowerCase() != 'a')
-		return;
+	var ae = document.activeElement, aet = '';
+	if (ae && ae != document.body)
+		aet = ae.nodeName.toLowerCase();
 
-	if (e.ctrlKey || e.altKey || e.metaKey || e.isComposing)
+	if (e.altKey || e.isComposing)
 		return;
 
 	if (QS('#bbox-overlay.visible'))
@@ -1796,6 +2090,55 @@ document.onkeydown = function (e) {
 
 	var k = e.code + '', pos = -1, n;
 
+	if (aet == 'tr' && ae.closest('#files')) {
+		var d = '';
+		if (k == 'ArrowUp') d = 'previous';
+		if (k == 'ArrowDown') d = 'next';
+		if (d) {
+			var el = ae[d + 'ElementSibling'];
+			if (el) {
+				el.focus();
+				if (ctrl(e))
+					document.documentElement.scrollTop += (d == 'next' ? 1 : -1) * el.offsetHeight;
+
+				if (e.shiftKey) {
+					clmod(el, 'sel', 't');
+					msel.selui();
+				}
+
+				return ev(e);
+			}
+		}
+		if (k == 'Space') {
+			clmod(ae, 'sel', 't');
+			msel.selui();
+			return ev(e);
+		}
+		if (k == 'KeyA' && ctrl(e)) {
+			var sel = msel.getsel(),
+				all = msel.getall();
+
+			msel.evsel(e, sel.length < all.length);
+			return ev(e);
+		}
+	}
+
+	if (aet && aet != 'a' && aet != 'tr')
+		return;
+
+	if (ctrl(e)) {
+		if (k == 'KeyX')
+			return fileman.cut();
+
+		if (k == 'KeyV')
+			return fileman.paste();
+
+		if (k == 'KeyK')
+			return fileman.delete();
+
+		return;
+	}
+
 	if (e.shiftKey && k != 'KeyA' && k != 'KeyD')
 		return;
 
@@ -1834,6 +2177,9 @@ document.onkeydown = function (e) {
 	if (k == 'KeyT')
 		return ebi('thumbs').click();
 
+	if (k == 'F2')
+		return fileman.rename();
+
 	if (!treectl.hidden && (!e.shiftKey || !thegrid.en)) {
 		if (k == 'KeyA')
 			return QS('#twig').click();
@@ -2118,7 +2464,6 @@ document.onkeydown = function (e) {
 		ebi('files').innerHTML = orig_html;
 		ebi('files').removeAttribute('q_raw');
 		orig_html = null;
-		msel.render();
 		reload_browser();
 	}
 })();
@@ -2135,6 +2480,7 @@ var treectl = (function () {
 		prev_atop = null,
 		prev_winh = null,
 		dyn = bcfg_get('dyntree', true),
+		dots = bcfg_get('dotfiles', false),
 		treesz = icfg_get('treesz', 16);
 
 	treesz = Math.min(Math.max(treesz, 4), 50);
@@ -2253,7 +2599,7 @@ var treectl = (function () {
 		xhr.dst = dst;
 		xhr.rst = rst;
 		xhr.ts = Date.now();
-		xhr.open('GET', dst + '?tree=' + top, true);
+		xhr.open('GET', dst + '?tree=' + top + (dots ? '&dots' : ''), true);
 		xhr.onreadystatechange = recvtree;
 		xhr.send();
 		enspin('#tree');
@@ -2264,7 +2610,7 @@ var treectl = (function () {
 			return;
 
 		if (this.status !== 200) {
-			alert("http " + this.status + ": " + this.responseText);
+			toast.err(0, "recvtree, http " + this.status + ": " + this.responseText);
 			return;
 		}
 
@@ -2357,7 +2703,7 @@ var treectl = (function () {
 		xhr.top = url;
 		xhr.hpush = hpush;
 		xhr.ts = Date.now();
-		xhr.open('GET', xhr.top + '?ls', true);
+		xhr.open('GET', xhr.top + '?ls' + (dots ? '&dots' : ''), true);
 		xhr.onreadystatechange = recvls;
 		xhr.send();
 		if (hpush)
@@ -2386,7 +2732,7 @@ var treectl = (function () {
 			return;
 
 		if (this.status !== 200) {
-			alert("http " + this.status + ": " + this.responseText);
+			toast.err(0, "recvls, http " + this.status + ": " + this.responseText);
 			return;
 		}
 
@@ -2453,7 +2799,6 @@ var treectl = (function () {
 
 		filecols.set_style();
 		mukey.render();
-		msel.render();
 		reload_tree();
 		reload_browser();
 
@@ -2495,6 +2840,13 @@ var treectl = (function () {
 		return ret;
 	}
 
+	function tdots(e) {
+		ev(e);
+		dots = !dots;
+		bcfg_set('dotfiles', dots);
+		treectl.goto(get_evpath());
+	}
+
 	function dyntree(e) {
 		ev(e);
 		dyn = !dyn;
@@ -2514,6 +2866,7 @@ var treectl = (function () {
 
 	ebi('entree').onclick = treectl.entree;
 	ebi('detree').onclick = treectl.detree;
+	ebi('dotfiles').onclick = tdots;
 	ebi('dyntree').onclick = dyntree;
 	ebi('twig').onclick = scaletree;
 	ebi('twobytwo').onclick = scaletree;
@@ -2558,9 +2911,22 @@ function despin(sel) {
 function apply_perms(newperms) {
 	perms = newperms || [];
 
-	ebi('acc_info').innerHTML = '<span>' + (acct != '*' ?
-		'<a href="/?pw=x">Logout ' + acct + '</a>' :
-		'<a href="/?h">Login</a>') + '</span>';
+	var axs = [],
+		aclass = '>',
+		chk = ['read', 'write', 'move', 'delete'];
+
+	for (var a = 0; a < chk.length; a++)
+		if (has(perms, chk[a]))
+			axs.push(chk[a].slice(0, 1).toUpperCase() + chk[a].slice(1));
+
+	axs = axs.join('-');
+	if (perms.length == 1) {
+		aclass = ' class="warn">';
+		axs += '-Only';
+	}
+
+	ebi('acc_info').innerHTML = '<span' + aclass + axs + ' access</span>' + (acct != '*' ?
+		'<a href="/?pw=x">Logout ' + acct + '</a>' : '<a href="/?h">Login</a>');
 
 	var o = QSA('#ops>a[data-perm], #u2footfoot');
 	for (var a = 0; a < o.length; a++) {
@@ -3007,44 +3373,79 @@ var arcfmt = (function () {
 
 
 var msel = (function () {
-	function getsel() {
-		var names = [],
-			links = QSA('#files tbody tr.sel td:nth-child(2) a');
+	var r = {};
+	r.sel = null;
+	r.all = null;
 
-		for (var a = 0, aa = links.length; a < aa; a++)
-			names.push(links[a].getAttribute('href').replace(/\/$/, "").split('/').slice(-1));
+	r.load = function () {
+		if (r.sel)
+			return;
 
-		return names;
-	}
-	function selui() {
-		clmod(ebi('wtoggle'), 'sel', getsel().length);
+		r.sel = [];
+		r.all = [];
+		var links = QSA('#files tbody td:nth-child(2) a:last-child'),
+			vbase = get_evpath();
+
+		for (var a = 0, aa = links.length; a < aa; a++) {
+			var href = links[a].getAttribute('href').replace(/\/$/, ""),
+				item = {};
+
+			item.id = links[a].getAttribute('id');
+			item.sel = links[a].closest('tr').classList.contains('sel');
+			item.vp = href.indexOf('/') !== -1 ? href : vbase + href;
+
+			r.all.push(item);
+			if (item.sel)
+				r.sel.push(item);
+
+			links[a].closest('tr').setAttribute('tabindex', '0');
+		}
+	};
+
+	r.getsel = function () {
+		r.load();
+		return r.sel;
+	};
+	r.getall = function () {
+		r.load();
+		return r.all;
+	};
+	r.selui = function () {
+		r.sel = r.all = null;
+		clmod(ebi('wtoggle'), 'sel', r.getsel().length);
 		thegrid.loadsel();
+		fileman.render();
 	}
-	function seltgl(e) {
+	r.seltgl = function (e) {
 		ev(e);
 		var tr = this.parentNode;
 		clmod(tr, 'sel', 't');
-		selui();
+		r.selui();
 	}
-	function evsel(e, fun) {
+	r.evsel = function (e, fun) {
 		ev(e);
 		var trs = QSA('#files tbody tr');
 		for (var a = 0, aa = trs.length; a < aa; a++)
 			clmod(trs[a], 'sel', fun);
-		selui();
+		r.selui();
 	}
 	ebi('selall').onclick = function (e) {
-		evsel(e, "add");
+		r.evsel(e, "add");
 	};
 	ebi('selinv').onclick = function (e) {
-		evsel(e, "t");
+		r.evsel(e, "t");
 	};
 	ebi('selzip').onclick = function (e) {
 		ev(e);
-		var names = getsel(),
+		var sel = r.getsel(),
 			arg = ebi('selzip').getAttribute('fmt'),
-			txt = names.join('\n'),
-			frm = mknod('form');
+			frm = mknod('form'),
+			txt = [];
+
+		for (var a = 0; a < sel.length; a++)
+			txt.push(vsplit(sel[a].vp)[1]);
+
+		txt = txt.join('\n');
 
 		frm.setAttribute('action', '?' + arg);
 		frm.setAttribute('method', 'post');
@@ -3064,16 +3465,17 @@ var msel = (function () {
 		console.log(txt);
 		frm.submit();
 	};
-	function render() {
+	r.render = function () {
 		var tds = QSA('#files tbody td+td+td');
 		for (var a = 0, aa = tds.length; a < aa; a++) {
-			tds[a].onclick = seltgl;
+			tds[a].onclick = r.seltgl;
 		}
+		r.selui();
 		arcfmt.render();
+		fileman.render();
+		ebi('selzip').style.display = ebi('unsearch') ? 'none' : '';
 	}
-	return {
-		"render": render
-	};
+	return r;
 })();
 
 
@@ -3100,6 +3502,160 @@ function ev_row_tgl(e) {
 }
 
 
+var unpost = (function () {
+	ebi('op_unpost').innerHTML = (
+		"you can delete your recent uploads below &ndash; click the fire-extinguisher icon to refresh" +
+		'<p>optional filter:&nbsp; URL must contain <input type="text" id="unpost_filt" size="20" /><a id="unpost_nofilt" href="#">clear filter</a></p>' +
+		'<div id="unpost"></div>'
+	);
+
+	var r = {},
+		ct = ebi('unpost'),
+		filt = ebi('unpost_filt');
+
+	r.files = [];
+	r.me = null;
+
+	r.load = function () {
+		var me = Date.now(),
+			html = [];
+
+		function unpost_load_cb() {
+			if (this.readyState != XMLHttpRequest.DONE)
+				return;
+
+			if (this.status !== 200) {
+				var msg = this.responseText;
+				toast.err(9, 'unpost-load failed:\n' + msg);
+				ebi('op_unpost').innerHTML = html.join('\n');
+				return;
+			}
+
+			var res = JSON.parse(this.responseText);
+			if (res.length) {
+				if (res.length == 2000)
+					html.push("<p>showing first 2000 files (use the filter)");
+				else
+					html.push("<p>" + res.length + " uploads can be deleted");
+
+				html.push(" &ndash; sorted by upload time &ndash; most recent first:</p>");
+				html.push("<table><thead><tr><td></td><td>time</td><td>size</td><td>file</td></tr></thead><tbody>");
+			}
+			else
+				html.push("<p>sike! no uploads " + (filt.value ? 'matching that filter' : '') + " are sufficiently recent</p>");
+
+			var mods = [1000, 100, 10];
+			for (var a = 0; a < res.length; a++) {
+				for (var b = 0; b < mods.length; b++)
+					if (a % mods[b] == 0 && res.length > a + mods[b] / 10)
+						html.push(
+							'<tr><td></td><td colspan="3" style="padding:.5em">' +
+							'<a me="' + me + '" class="n' + a + '" n2="' + (a + mods[b]) +
+							'" href="#">delete the next ' + Math.min(mods[b], res.length - a) + ' files below</a></td></tr>');
+				html.push(
+					'<tr><td><a me="' + me + '" class="n' + a + '" href="#">delete</a></td>' +
+					'<td>' + unix2iso(res[a].at) + '</td>' +
+					'<td>' + res[a].sz + '</td>' +
+					'<td>' + linksplit(res[a].vp).join(' ') + '</td></tr>');
+			}
+
+			html.push("</tbody></table>");
+			ct.innerHTML = html.join('\n');
+			r.files = res;
+			r.me = me;
+		}
+
+		var q = '/?ups';
+		if (filt.value)
+			q += '&filter=' + uricom_enc(filt.value, true);
+
+		var xhr = new XMLHttpRequest();
+		xhr.open('GET', q, true);
+		xhr.onreadystatechange = unpost_load_cb;
+		xhr.send();
+
+		ct.innerHTML = "<p><em>loading your recent uploads...</em></p>";
+	};
+
+	function unpost_delete_cb() {
+		if (this.readyState != XMLHttpRequest.DONE)
+			return;
+
+		if (this.status !== 200) {
+			var msg = this.responseText;
+			toast.err(9, 'unpost-delete failed:\n' + msg);
+			return;
+		}
+
+		for (var a = this.n; a < this.n2; a++) {
+			var o = QSA('#op_unpost a.n' + a);
+			for (var b = 0; b < o.length; b++) {
+				var o2 = o[b].closest('tr');
+				o2.parentNode.removeChild(o2);
+			}
+		}
+		toast.ok(5, this.responseText);
+
+		if (!QS('#op_unpost a[me]'))
+			ebi(goto_unpost());
+	}
+
+	ct.onclick = function (e) {
+		var tgt = e.target.closest('a[me]');
+		if (!tgt)
+			return;
+
+		if (!tgt.getAttribute('href'))
+			return;
+
+		var ame = tgt.getAttribute('me');
+		if (ame != r.me)
+			return toast.err(0, 'something broke, please try a refresh');
+
+		var n = parseInt(tgt.className.slice(1)),
+			n2 = parseInt(tgt.getAttribute('n2') || n + 1),
+			req = [];
+
+		for (var a = n; a < n2; a++)
+			if (QS('#op_unpost a.n' + a))
+				req.push(r.files[a].vp);
+
+		var links = QSA('#op_unpost a.n' + n);
+		for (var a = 0, aa = links.length; a < aa; a++) {
+			links[a].removeAttribute('href');
+			links[a].innerHTML = '[busy]';
+		}
+
+		toast.inf(0, "deleting " + req.length + " files...");
+
+		var xhr = new XMLHttpRequest();
+		xhr.n = n;
+		xhr.n2 = n2;
+		xhr.open('POST', '/?delete', true);
+		xhr.onreadystatechange = unpost_delete_cb;
+		xhr.send(JSON.stringify(req));
+	};
+
+	var tfilt = null;
+	filt.oninput = function () {
+		clearTimeout(tfilt);
+		tfilt = setTimeout(r.load, 250);
+	};
+
+	ebi('unpost_nofilt').onclick = function () {
+		filt.value = '';
+		r.load();
+	};
+
+	return r;
+})();
+
+
+function goto_unpost(e) {
+	unpost.load();
+}
+
+
 function reload_mp() {
 	if (mp && mp.au) {
 		mp.au.pause();
@@ -3148,8 +3704,8 @@ function reload_browser(not_mp) {
 		up2k.set_fsearch();
 
 	thegrid.setdirty();
+	msel.render();
 }
 reload_browser(true);
 mukey.render();
-msel.render();
 play_linked();
